#define ERR_PARAMETRES 1
#define ERR_NO_MOUNT_OPT 2
#define ERR_NO_REPO_OPT 3

#define ERR_UNKNOWN_OPTION 4

#define ERR_NO_MOUNT 5
#define ERR_NO_REPO 6
#define ERR_NO_TMP 8

#define ERR_RDIFF 7

#define ERR_FULL_ONLY 9
#define ERR_REPO_READ 10

void fail(int);

void clean_up();
