#ifndef _INITIALIZE_H_
#define _INITIALIZE_H_

#include "headers.h"
#include "errors.h"
#include "externs.h"
#include "support/gstring.h"
#include "support/gpath.h"
#include "operations.h"

#include "structure/core.h"
#include "structure/full.h"
#include "structure/necessary.h"

#include "layout/core.h"
#include "layout/versions.h"
#include "layout/all.h"

#include "retriever/retriever.h"

void initialize(struct file_system_info *fsinfo);

#endif
